//
//  YSBubbleControl.m
//  YSBubbleControl
//
//  Created by yaoshuai on 2017/1/13.
//  Copyright © 2017年 ys. All rights reserved.
//

#import "YSBubbleControl.h"

@interface YSBubbleControl()

@property (nonatomic,strong) UIView *frontView;

// 气泡上显示数字的label
@property (nonatomic,strong) UILabel *bubbleLabel;

// 父视图
@property (nonatomic,strong) UIView *superView;

// 气泡颜色
@property (nonatomic,strong) UIColor *bubbleColor;

// 气泡粘性系数，越大可以拉得越长
@property (nonatomic,assign) CGFloat viscosity;

@end

@implementation YSBubbleControl{
    UIBezierPath *cutePath;
    UIColor *fillColorForCute;
    UIDynamicAnimator *animator;
    UISnapBehavior  *snap;
    
    UIView *backView;
    CGFloat r1;
    CGFloat r2;
    CGFloat x1;
    CGFloat y1;
    CGFloat x2;
    CGFloat y2;
    CGFloat centerDistance;
    CGFloat cosDigree;
    CGFloat sinDigree;
    
    CGPoint pointA;
    CGPoint pointB;
    CGPoint pointD;
    CGPoint pointC;
    CGPoint pointO;
    CGPoint pointP;
    
    CGRect oldBackViewFrame;
    CGPoint oldBackViewCenter;
    CAShapeLayer *shapeLayer;
}

@synthesize messageCount = _messageCount;

// MARK: - 初始化
- (instancetype)initWithFrame:(CGRect)frame andSuperView:(UIView *)view andFillColor:(UIColor *)fillColor{
    if (self = [super initWithFrame:frame]) {
        _superView = view;
        _bubbleColor = fillColor;
        _viscosity = 20;
        
        [_superView addSubview:self];
        
        [self setupUI];
    }
    return self;
}

-(void)setupUI{
    shapeLayer = [CAShapeLayer layer];
    self.backgroundColor = [UIColor clearColor];
    self.frontView = [[UIView alloc] initWithFrame:self.frame];
    
    r2 = self.frontView.bounds.size.width / 2;
    self.frontView.layer.cornerRadius = r2;
    self.frontView.backgroundColor = self.bubbleColor;
    
    backView = [[UIView alloc] initWithFrame:self.frontView.frame];
    r1 = backView.bounds.size.width / 2;
    backView.layer.cornerRadius = r1;
    backView.backgroundColor = self.bubbleColor;
    
    self.bubbleLabel = [[UILabel alloc] init];
    self.bubbleLabel.frame = CGRectMake(0, 0, self.frontView.bounds.size.width, self.frontView.bounds.size.height);
    self.bubbleLabel.textColor = [UIColor whiteColor];
    self.bubbleLabel.textAlignment = NSTextAlignmentCenter;
    
    [self.frontView insertSubview:self.bubbleLabel atIndex:0];
    
    [self.superView addSubview:backView];
    [self.superView addSubview:self.frontView];
    
    x1 = backView.center.x;
    y1 = backView.center.y;
    x2 = self.frontView.center.x;
    y2 = self.frontView.center.y;
    
    pointA = CGPointMake(x1-r1,y1);
    pointB = CGPointMake(x1+r1, y1);
    pointD = CGPointMake(x2-r2, y2);
    pointC = CGPointMake(x2+r2, y2);
    pointO = CGPointMake(x1-r1,y1);
    pointP = CGPointMake(x2+r2, y2);
    
    oldBackViewFrame = backView.frame;
    oldBackViewCenter = backView.center;
    
    backView.hidden = YES;//为了看到 frontView 的气泡晃动效果，需要暂时隐藏 backView
    [self addAniamtionLikeGameCenterBubble];
    
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc]initWithTarget:self action:@selector(handleDragGesture:)];
    [self.frontView addGestureRecognizer:pan];
}

// MARK: - 属性
- (NSUInteger)messageCount{
    return MAX(_messageCount, 0);
}
- (void)setMessageCount:(NSUInteger)messageCount{
    messageCount = MAX(messageCount, 0);
    self.bubbleLabel.text = @(messageCount).description;
    self.frontView.hidden = messageCount <= 0;
    
    if (messageCount <= 0){
        self.frontView.center = oldBackViewCenter;
        [self drawRect];
        [self RemoveAniamtionLikeGameCenterBubble];
    }
    else{
        if (self.frontView.layer.animationKeys.count == 0){
            [self addAniamtionLikeGameCenterBubble];
        }
    }
    
    _messageCount = messageCount;
}

// MARK: - 绘图
-(void)drawRect{
    x1 = backView.center.x;
    y1 = backView.center.y;
    x2 = self.frontView.center.x;
    y2 = self.frontView.center.y;
    
    centerDistance = sqrtf((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
    if (centerDistance == 0) {
        cosDigree = 1;
        sinDigree = 0;
    }else{
        cosDigree = (y2-y1)/centerDistance;
        sinDigree = (x2-x1)/centerDistance;
    }
    
    r1 = oldBackViewFrame.size.width / 2 - centerDistance/self.viscosity;
    
    pointA = CGPointMake(x1-r1*cosDigree, y1+r1*sinDigree);  // A
    pointB = CGPointMake(x1+r1*cosDigree, y1-r1*sinDigree); // B
    pointD = CGPointMake(x2-r2*cosDigree, y2+r2*sinDigree); // D
    pointC = CGPointMake(x2+r2*cosDigree, y2-r2*sinDigree);// C
    pointO = CGPointMake(pointA.x + (centerDistance / 2)*sinDigree, pointA.y + (centerDistance / 2)*cosDigree);
    pointP = CGPointMake(pointB.x + (centerDistance / 2)*sinDigree, pointB.y + (centerDistance / 2)*cosDigree);
    
    backView.center = oldBackViewCenter;
    backView.bounds = CGRectMake(0, 0, r1*2, r1*2);
    backView.layer.cornerRadius = r1;
    
    cutePath = [UIBezierPath bezierPath];
    [cutePath moveToPoint:pointA];
    [cutePath addQuadCurveToPoint:pointD controlPoint:pointO];
    [cutePath addLineToPoint:pointC];
    [cutePath addQuadCurveToPoint:pointB controlPoint:pointP];
    [cutePath moveToPoint:pointA];
    
    if (backView.hidden == NO) {
        shapeLayer.path = [cutePath CGPath];
        shapeLayer.fillColor = [fillColorForCute CGColor];
        [self.superView.layer insertSublayer:shapeLayer below:self.frontView.layer];
    }
    
}



// MARK: - 处理拖拽手势
-(void)handleDragGesture:(UIPanGestureRecognizer *)ges{
    // CGPoint dragPoint = [ges locationInView:self.superView];
    CGPoint dragPoint = dragPoint = [ges locationInView: [UIApplication sharedApplication].keyWindow];
    
    if (ges.state == UIGestureRecognizerStateBegan) {
        backView.hidden = NO;
        fillColorForCute = self.bubbleColor;
        [self RemoveAniamtionLikeGameCenterBubble];
        
    }else if (ges.state == UIGestureRecognizerStateChanged){
        self.frontView.center = dragPoint;
        if (r1 <= 6) {
            fillColorForCute = [UIColor clearColor];
            backView.hidden = YES;
            [shapeLayer removeFromSuperlayer];
        }
        [self drawRect];
    } else if (ges.state == UIGestureRecognizerStateEnded || ges.state ==UIGestureRecognizerStateCancelled || ges.state == UIGestureRecognizerStateFailed){
        
        backView.hidden = YES;
        fillColorForCute = [UIColor clearColor];
        [shapeLayer removeFromSuperlayer];
        if(r1 <= 6)
        {
            self.messageCount = 0;
            [self sendActionsForControlEvents:UIControlEventTouchDragExit];
        }
        else{
            [UIView animateWithDuration:0.5 delay:0.0f usingSpringWithDamping:0.4f initialSpringVelocity:0.0f options:UIViewAnimationOptionCurveEaseInOut animations:^{
                self.frontView.center = oldBackViewCenter;
            } completion:^(BOOL finished) {
                if (finished) {
                    [self addAniamtionLikeGameCenterBubble];
                }
            }];
        }
    }
}

// MARK: - 添加类似GameCenter的气泡晃动动画
-(void)addAniamtionLikeGameCenterBubble {
    
    CAKeyframeAnimation *pathAnimation = [CAKeyframeAnimation animationWithKeyPath:@"position"];
    pathAnimation.calculationMode = kCAAnimationPaced;
    pathAnimation.fillMode = kCAFillModeForwards;
    pathAnimation.removedOnCompletion = NO;
    pathAnimation.repeatCount = INFINITY;
    pathAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
    pathAnimation.duration = 5.0;
    
    CGMutablePathRef curvedPath = CGPathCreateMutable();
    CGRect circleContainer = CGRectInset(self.frontView.frame, self.frontView.bounds.size.width / 2 - 3, self.frontView.bounds.size.width / 2 - 3);
    CGPathAddEllipseInRect(curvedPath, NULL, circleContainer);
    
    pathAnimation.path = curvedPath;
    CGPathRelease(curvedPath);
    [self.frontView.layer addAnimation:pathAnimation forKey:@"myCircleAnimation"];
    
    CAKeyframeAnimation *scaleX = [CAKeyframeAnimation animationWithKeyPath:@"transform.scale.x"];
    scaleX.duration = 1;
    scaleX.values = @[@1.0, @1.1, @1.0];
    scaleX.keyTimes = @[@0.0, @0.5, @1.0];
    scaleX.repeatCount = INFINITY;
    scaleX.autoreverses = YES;
    
    scaleX.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    [self.frontView.layer addAnimation:scaleX forKey:@"scaleXAnimation"];
    
    CAKeyframeAnimation *scaleY = [CAKeyframeAnimation animationWithKeyPath:@"transform.scale.y"];
    scaleY.duration = 1.5;
    scaleY.values = @[@1.0, @1.1, @1.0];
    scaleY.keyTimes = @[@0.0, @0.5, @1.0];
    scaleY.repeatCount = INFINITY;
    scaleY.autoreverses = YES;
    scaleY.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    [self.frontView.layer addAnimation:scaleY forKey:@"scaleYAnimation"];
}

// MARK: - 移除动画
-(void)RemoveAniamtionLikeGameCenterBubble {
    [self.frontView.layer removeAllAnimations];
}

@end
